const path = require('path');
const fs = require('fs');
const { Gitlab } = require('@gitbeaker/rest');
const config = require('./config');

const FULL_GROUP_NAME = `${config.gitlabHost}/${config.groupName}/`;
const ACCESS_LEVEL = 30;

const STUDENTS_FILE_NAME = "../data/students.txt";
const studentsFilePath = path.join(__dirname, STUDENTS_FILE_NAME);

const api = new Gitlab({
  token: config.gitlabToken,
  host: config.gitlabHost,
});

function readStudentsFile() {
  if (!fs.existsSync(studentsFilePath)) {
    console.error(`File ${studentsFilePath} doesn't exist`);
    return [];
  }
  const data = fs.readFileSync(studentsFilePath, 'utf8');
  return data.split('\n').map(line => {
    const parts = line.split('\t');
    return {
      name: `${parts[1]} ${parts[0]}`,
      slug: parts[2]?.replace(/[\r\n\t]/g, ''),
    };
  });
}

async function getProject(slug, groupId) {
  const projects = await api.Projects.search(slug);
  return projects.filter(project => project.namespace.id === groupId)[0];
}

async function addProjectToGroup(name, slug, groupId) {
    return api.Projects.create({
      name,
      visibility: 'private',
      namespaceId: groupId,
      path: slug,
    });
}

async function getUserByName(name) {
    const [user] = await api.Users.all({search: name});
    if (user) {
      return user;
    }
}

async function getUserBySlug(slug) {
  const [user] = await api.Users.all({username: slug});
  if (user) {
    return user;
  }

  const [user1] = await api.Users.all({username: `${slug}1`});
  return user1;
}

async function addUserToProject(userId, projectId) {
  const members = await api.ProjectMembers.all(projectId);
  const foundUser = members.find(member => member.id === userId);
  if (!foundUser) {
    await api.ProjectMembers.add(projectId, ACCESS_LEVEL, { userId });
    console.log("Added user to project member");
    return;
  }
  console.debug("User is already member of project");
}

async function main() {
    console.info(`Start sync ${new Date().toISOString()}`);

    const group = await api.Groups.show(config.groupName);
    if (!group) {
      console.error(`Group ${config.groupName} in ${config.gitlabHost} not found. Create it before running this service.`);
      return;
    }

    const students = readStudentsFile();
    for (const student of students) {
      try {
        const project = await getProject(student.slug, group.id) ||  await addProjectToGroup(student.name, student.slug, group.id);
        const userBySlug = await getUserBySlug(student.slug);
        const userByName = await getUserByName(student.name);
        const user = userBySlug || userByName;

        if (!user) {
          console.warn(`Can't find user ${student.slug}`);
          continue;
        }

        console.debug(`Adding user ${student.name} to project ${project.web_url}`);

        await addUserToProject(user.id, project.id);
      } catch(err) {
        console.error(err);
      }
    }
}

console.log("Start program");

main().catch(err => console.error(err));

setInterval(() => {
  main().catch(err => console.error(err));
}, config.timeout);
